/*

University of Sao Paulo - Institute of Mathematics and Computer Science ( ICMC - USP )
Computer Graphics - SCC 0250
Developer (PAE): Vinicius R. P. Borges
Profa: Maria Cristina Ferreira de Oliveira

Codigo 5 - Aula de Laboratorio 3 - 09/10/2012
CODE 5 - Lab 3

> Para compilar este codigo, execute no terminal do Linux - To compile this code, type the following command on Linux Terminal

# gcc escopoTransfGeom.c  -lX11 -lglut -lGL -lGLU -lm -o escopoTransfGeom

Para executar o codigo, digite - To run this code, type:

# ./escopoTransfGeom


ATENÇÃO: Para a correta utilizacao do codigo,
         siga as instrucoes contidas na funcao DrawGLScene
         para seu correto funcionamento e entendimento
*/

//Bibliotecas - Libraries

// Biblioteca de entrada e saida padrao de dados
#include <stdio.h>

#include <stdlib.h>

// Biblioteca GLUT : Criacao de janelas e tratamento de eventos de forma independente do sistema operacional
#include <GL/glut.h>

// Estrutura desenvolvida para armazenar vertices de um poligono
struct vertice
{
    GLfloat x; //ordenada x do vertice
    GLfloat y; //ordenada y do vertice
};

typedef struct vertice Vertice;

struct poligono
{
    Vertice *vertices; //poligono eh definido pelo conjunto de vertices
    GLint nroVertices; //quantidade de vertices
};

typedef struct poligono Poligono;

// Armazena valor inteiro que identifica uma janela
int window;

// Variavel que armazena o poligono do programa
Poligono *triangle;
Poligono *square;

// Funcao que cria e aloca um poligono na memoria
Poligono* criaPoligono(GLfloat *vertices, int tam)
{
          Poligono *novo;
          GLint i;
          GLint corr;
          
          novo = (Poligono *) malloc(sizeof(Poligono));
          
          novo->nroVertices = (int) tam/2;
          
          novo->vertices = (Vertice *) malloc (sizeof(Vertice)*novo->nroVertices);

          for(i = 0; i < tam; i = i+2){
               corr = i/2;
               novo->vertices[corr].x = vertices[i];
               novo->vertices[corr].y = vertices[i+1];
          }
          
          return novo;
}

// Função que calcula o centroide de um poligono
Vertice calcCentroide(Poligono *p)
{
        GLint i;
        Vertice centroide;
        
        centroide.x = 0.0f;
        centroide.y = 0.0f;
        
        for(i = 0; i < p->nroVertices;i++)
        {
              centroide.x = centroide.x + p->vertices[i].x;
              centroide.y = centroide.y + p->vertices[i].y;
        }
        
        //centroide.x /= p->nroVertices;
        centroide.x = centroide.x / p->nroVertices;
        centroide.y = centroide.y / p->nroVertices;
        
        return centroide;
}

// Funcao que desenha o poligono na tela
void desenhaPoligono(Poligono *p){
     GLint i;
     
     glBegin(GL_TRIANGLE_FAN);
     
         for( i = 0; i < p->nroVertices ; i++)
         {
              glVertex2f(p->vertices[i].x,p->vertices[i].y);
         }
     
     glEnd();
}

// Funcao que prepara a janela onde a cena grafica sera desenhada
void InitGL(int Width, int Height)
{
     GLfloat vertTriangle[] = { 0.2f, 0.2f, 0.4f, 0.2f, 0.3f, 0.4f };
     GLfloat vertSquare[] = {-0.3f, -0.1f, -0.3f, -0.3f, -0.1f, -0.3f,-0.1f,-0.1f};
     
     //Ajusta a cor de fundo para branco
     glClearColor(1.0f,1.0f,1.0f,0.0f);
     
     //Carrega a matriz de projecao
     glMatrixMode(GL_PROJECTION);
     
     //Define projecao ortogonal que mapeia o poligono nas coordenadas do mundo para coordenadas da tela
     gluOrtho2D(-1.0,1.0,-1.0,1.0);
     
     // Aloca os poligonos a serem utilizados no programa
     triangle = criaPoligono(vertTriangle,6);
     square = criaPoligono(vertSquare,8);
}

void DrawGLScene()
{
     // Variaveis que armazenam os centroides de cada poligono da cena
     Vertice centroide_triangle;
     Vertice centroide_square;
     
     // Calcula os centroides
     centroide_triangle = calcCentroide(triangle);
     centroide_square = calcCentroide(square);
     
     // Muda para o sistema de coordenadas do modelo
     glMatrixMode(GL_MODELVIEW);
     
     // Inicializa a matriz de transformação corrente
     glLoadIdentity();
     
     // Limpa a tela e o Depth Buffer
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
     
     // Ajusta a cor do poligono
     glColor3f(1.0,0.0,0.0);
     
     // Empilha o estado atual da matriz de transformacoes
     glPushMatrix();
     
     // Tres comando para se rotacionar o sistema de coordenadas da tela em 45 graus
     glTranslatef(centroide_triangle.x,centroide_triangle.y,0.0f);
     
     glRotatef(45.0f,0.0f,0.0f,1.0f);
     
     glTranslatef(-centroide_triangle.x,-centroide_triangle.y,0.0f);
     
     // Desenha o triangulo no plano rotacionado em 45 graus
     desenhaPoligono(triangle);
     
     // Desfaz a rotacao de 45 graus do sistema de coordenadas  
     glPopMatrix();
     
     // Muda a cor do proximo objeto a ser desenhado
     glColor3f(0.0,1.0,0.0);
     
     // Desenha o poligono no sistema de coordenadas original
     desenhaPoligono(square);
     
     // Processa as rotinas openGL o mais rapido possivel
     glFlush();
     
     // O que se obtém então é somente o triangulo rotacionado em 45 graus e o quadrado sem rotacao alguma
}

// Funcao principal
main(int argc, char *argv[])
{
         
    //Inicializa o Glut
    glutInit(&argc, argv);

    // Ajusta o display para o usuario
    glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH);

    //Coordenada inicial da janela
    glutInitWindowPosition(0, 0);

    //Ajusta o tamanho da janela com 640x480
    glutInitWindowSize(640, 480);
    
    //Nome da Janela  
    window = glutCreateWindow("Escopo de Transformacoes Geometricas 2D");

    //Funcao que prepara a cena grafica na janela com dimensoes 640 x 480
    InitGL(640, 480);

    //Desenha a cena grafica na tela
    glutDisplayFunc(DrawGLScene);
    
    //Loop de atualizacao da tela
    glutMainLoop();
}

